<?php
/**
 * WooCommerce Gateways By Location
 *
 * @author 		StoreApps
 * @category 	Admin
 * @package 	WooCommerce Gateways By Location
 * @version     1.2.9
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'SA_WC_Gateways_By_Location' ) ) {

	/**
	 * Class SA_WC_Gateways_By_Location
	 */
	class SA_WC_Gateways_By_Location {

		private $version;
		private $plugin_data;
		private $plugin_url;
		private $plugin_file;

		function __construct( $file ) {

			$this->plugin_file = $file;
			$this->plugin_url = plugins_url( '', dirname(__FILE__) );

			add_action( 'admin_init', array( $this, 'get_plugin_data' ) );
			add_filter( 'woocommerce_available_payment_gateways', array( $this, 'display_gateway_by_location' ) );

			// Decide which user roles whill have access to GBL options - Administrator, Shop Manager and who can manage options
			if ( is_admin() || current_user_can( 'manage_woocommerce' ) || current_user_can( 'manage_options' ) ) {	
				add_action( 'admin_enqueue_scripts', array( $this, 'load_js_css' ) );
				add_action( 'woocommerce_admin_field_payment_gateways', array( $this, 'gateway_by_location_output' ), 20 );
				add_action( 'woocommerce_update_options_checkout', array( $this, 'save' ), 1000 );
			}

			add_filter( 'plugin_action_links_' . plugin_basename( $this->plugin_file ), array( $this, 'plugin_action_links' ) );

			if ( ! $this->is_wc_gte_25() ) {
				add_action( 'admin_notices', array( $this, 'admin_notice_sa_needs_wc_25_above' ) );
			}

		}

		/**
		 * to handle WC compatibility related function call from appropriate class
		 * 
		 * @param $function_name string
		 * @param $arguments array of arguments passed while calling $function_name
		 * @return result of function call
		 * 
		 */
		public function __call( $function_name, $arguments = array() ) {

			if ( ! is_callable( 'SA_WC_Compatibility_3_0', $function_name ) ) return;

			if ( ! empty( $arguments ) ) {
				return call_user_func_array( 'SA_WC_Compatibility_3_0::'.$function_name, $arguments );
			} else {
				return call_user_func( 'SA_WC_Compatibility_3_0::'.$function_name );
			}

		}

		function get_plugin_data() {

			if( empty( $this->plugin_file ) ) {
				return;
			}

			$this->plugin_data = get_plugin_data( $this->plugin_file );
			$this->version = $this->plugin_data['Version'];
			
		}

		function load_js_css() {

			if( version_compare(WC()->version, '2.3.0', '>=') ) {
				wp_register_script( 'sa_gateways_by_location_chosen', $this->plugin_url . '/assets/js/chosen.jquery.min.js', array( 'jquery' ), $this->version );	
			}
			wp_register_script( 'sa_gateways_by_location_js', $this->plugin_url . '/assets/js/admin.js', array( 'jquery' ), $this->version );
			wp_enqueue_style( 'sa_gateways_by_location_css', $this->plugin_url . '/assets/css/admin.css', array(), $this->version );
			wp_enqueue_style( 'dashicons' );
		}	

		function gateway_by_location_output( $settings ) {

			// Enqueue chosen when WC 2.3 or above
			if(version_compare(WC()->version, '2.3.0', '>=')) {
				if ( ! wp_script_is( 'sa_gateways_by_location_chosen' ) ) {
					wp_enqueue_script( 'sa_gateways_by_location_chosen' );
				}
			}

			if( ! wp_script_is( 'sa_gateways_by_location_js' ) ) {
				wp_enqueue_script( 'sa_gateways_by_location_js' );
			}
				
			$available_gateways = (array) WC()->payment_gateways->payment_gateways();
			$gateway_locations = get_option( 'sa_wc_gateways_by_location', null );
			$gateway_locations_search_in = get_option( 'sa_wc_gateways_by_location_search_in', null );
			if( empty( $gateway_locations_search_in ) ) {
				$gateway_locations_search_in['address'] = 'billing';
				update_option( 'sa_wc_gateways_by_location_search_in', $gateway_locations_search_in );
			}
			
			?>
			
			<tr valign="top">
				<th scope="row" class="titledesc"><?php echo __( 'Gateway By Location', 'woocommerce-gateways-by-location' ) ?></th>
				<td class="forminp gateway_by_location">
					<table id="gateway_by_location" class="wc_gateways_by_location widefat" cellspacing="0">
					<thead>
					<tr>
						<td>
							<span class='search_in'><label><?php echo __( 'Address to look in', 'woocommerce-gateways-by-location' ) ?></label></span>
						</td>
						<td>
							<label for="billing">
									<input type="radio" name="sa_wc_gateways_by_location_search_in[address]" value="billing" <?php ( !empty( $gateway_locations_search_in['address'] ) ) ? checked( $gateway_locations_search_in['address'], 'billing' ) : ''; ?> />
										<?php _e( 'Billing', 'woocommerce-gateways-by-location' ); ?>
							</label> &nbsp;
							<label for="shipping">
									<input type="radio" name="sa_wc_gateways_by_location_search_in[address]" value="shipping" <?php ( !empty( $gateway_locations_search_in['address'] ) ) ? checked( $gateway_locations_search_in['address'], 'shipping' ) : ''; ?> />
										<?php _e( 'Shipping', 'woocommerce-gateways-by-location' ); ?>
							</label>
						</td>	
					</tr>
					<tr>
						<?php
							$columns = array(
								'gateway_name'  => __( 'Gateway', 'woocommerce-gateways-by-location' ),
								'available'     => __( 'Avaliable Only In', 'woocommerce-gateways-by-location' ),
								'not_available' => __( 'Not Avaliable In', 'woocommerce-gateways-by-location' ),
								'action'		=> __('Action', 'woocommerce-gateways-by-location'),
							);
							foreach ( $columns as $key => $column ) {
								if( esc_attr( $key ) == 'action'){
									echo '<th class="' . esc_attr( $key ) . '" colspan="2">' . esc_html( $column ) . '</th>';
								} else {
									echo '<th class="' . esc_attr( $key ) . '">' . esc_html( $column ) . '</th>';
								}
							}
						?>
					</tr>
					<tbody>
						<?php
							echo '<tr class="location">';
							echo '<td>';
							echo '<select id="gateway_list">';
							echo '<option value="select">'.__( 'Select Gateway', 'woocommerce-gateways-by-location' ).'</option>';
								foreach ($available_gateways as $gateway) {
									if ( $gateway->enabled == 'yes' ) {
										echo '<option value="' . $gateway->id . '">' . esc_html( $gateway->title ) . '</option>';											
									}
								}
							echo '</select>';							
							echo '</td>';
							echo '<td>';
							echo '<select id="avail_in" name="sa_wc_gateways_by_location['.$gateway->id.'"][available_in]" data-placeholder="' . __( 'Select Countries', 'woocommerce-gateways-by-location' ) .'" multiple="multiple" class="sa_gbl_location" >';
								foreach( WC()->countries->countries as $code => $country ){
									echo '<option value="' . esc_html( $code ) . '">' . $country . '</option>';
								
								}
							echo '</select></td>';
							echo '<td>';
							echo '<select id="not_avail_in" name="sa_wc_gateways_by_location['.$gateway->id.'"][not_available_in]" data-placeholder="' . __( 'Select Countries', 'woocommerce-gateways-by-location' ) .'" multiple="multiple" class="sa_gbl_location" >';
								foreach( WC()->countries->countries as $code => $country ){
									echo '<option value="' . esc_html( $code ) . '">' . $country . '</option>';
								}						
							echo '</select></td>';
							echo '<td colspan="2"><a class="button add_row">'.__( 'Add Rule', 'woocommerce-gateways-by-location' ).'</a></td>';
							echo '</tr>';
								
							if( !empty( $gateway_locations ) && is_array( $gateway_locations )){
								foreach ($gateway_locations as $key => $value) {
									$avail_list = $not_avail_list = array();
									echo '<tr><td>'. esc_html( $value['title'] ) . '</td>';
									$avail_in = array_filter(explode(',', $value['available_in'] ));
									$not_avail_in = array_filter(explode(',', $value['not_available_in'] ));
									
									if( !empty( $avail_in ) ){
										for ( $i=0; $i < count( $avail_in ); $i++ ) { 
											$available_countries = WC()->countries->countries[$avail_in[$i]];
											if ( !empty( $available_countries ) ) {
												$avail_list[] = $available_countries;
											}
										}
										echo '<td>' . implode(', ', $avail_list) . '</td>';
									} else {
										echo '<td>'.__( 'All', 'woocommerce-gateways-by-location' ).'</td>';
									}
									
									if( !empty( $not_avail_in ) ) {
										for ($j=0; $j < count( $not_avail_in ); $j++) {
											$not_available_countries = WC()->countries->countries[$not_avail_in[$j]];
											if ( !empty( $not_available_countries ) ) {
												$not_avail_list[] = $not_available_countries;
											}
										}
										echo '<td>' . implode(', ', $not_avail_list) . '</td>';
									} else {
										echo '<td>'.__('-', 'woocommerce-gateways-by-location') .'</td>';
									}

									echo '<td class="edit_row" title="'.__('Edit Rule', 'woocommerce-gateways-by-location') .'"></td>' ;
									echo '<td class="delete_row" title="'.__('Delete Rule', 'woocommerce-gateways-by-location') .'"></td>' ;
									echo '<input type="hidden" class="gateway_id" name="sa_wc_gateways_by_location[' . $key . '][title]" value="' . esc_html( $value['title'] ) . '">';
									echo '<input type="hidden" class="avail_in" name="sa_wc_gateways_by_location[' . $key . '][available_in]" value="' . esc_html( $value['available_in'] ) . '">';
									echo '<input type="hidden" class="not_avail_in" name="sa_wc_gateways_by_location[' . $key . '][not_available_in]" value="' . esc_html( $value['not_available_in'] ) . '">';
									echo '</tr>';
								}
							}
						?>
					</tbody>
				</table>
				<p><em><?php echo __( 'Not adding rule for a gateway will show it for all countries.', 'woocommerce-gateways-by-location' ); ?></em></p>
				</td>
			</tr>		
			<?php
		}

		function display_gateway_by_location( $payment_gateways ) {
			
			if( isset( $_GET['page'] ) && $_GET['page'] == 'wc-settings') {
				return $payment_gateways;
			}

			$gateway_locations = get_option( 'sa_wc_gateways_by_location', null );
			$gateway_locations_search_in = get_option( 'sa_wc_gateways_by_location_search_in', null );

			$customer = WC()->customer;

			if ( ! empty( $customer ) ) {

				if ( ! $customer instanceof WC_Customer ) {
					WC()->customer = new WC_Customer();
				}

				$called_by = current_filter();		// check why it is used

				// Collect Customer country
				if( ! empty( $gateway_locations_search_in ) && $gateway_locations_search_in['address'] == 'billing' ){
					if ( $this->is_wc_gte_30() ) {
						$country = ( is_object( WC()->customer ) && is_callable( array( WC()->customer, 'get_billing_country' ) ) ) ? WC()->customer->get_billing_country() : '';
					} else {
						$customer_country = WC()->customer->country;
						$country = ( ! empty( $customer_country ) ) ? $customer_country : WC()->customer->get_country();
					}
				} else {
					if ( $this->is_wc_gte_30() ) {
						$country = ( is_object( WC()->customer ) && is_callable( array( WC()->customer, 'get_shipping_country' ) ) ) ? WC()->customer->get_shipping_country() : '';
					} else {
						$customer_shipping_country = WC()->customer->shipping_country;
						$country = ( ! empty( $customer_shipping_country ) ) ? $customer_shipping_country : WC()->customer->get_shipping_country();
					}
				}

				if ( ! empty( $country ) && ! empty( $gateway_locations ) && is_array( $gateway_locations ) ) {
					foreach ( $gateway_locations as $code => $location_rules ) {
						if( !empty ( $location_rules ) ) {
								if( 
									( !empty( $location_rules['available_in'] ) && strpos( ",{$location_rules['available_in']},",  ",{$country}," ) === false ) ||
									( !empty($location_rules['not_available_in']) && strpos(  ",{$location_rules['not_available_in']},",  ",{$country}," ) !== false )
								) {
									unset( $payment_gateways[ $code ] );
								}
						}
					}
				}
			}

			return $payment_gateways;
		}

		function save() {
			if( empty( $_GET['page'] ) || $_GET['page'] != 'wc-settings' || empty($_GET['tab']) || $_GET['tab'] != 'checkout' || !empty($_GET['section']) ) {
				return;			
			}

			if ( array_key_exists('sa_wc_gateways_by_location', $_POST) ) {

				$gateway_by_locations = $_POST['sa_wc_gateways_by_location'];			
				if( !empty( $gateway_by_locations ) && is_array( $gateway_by_locations ) ){
					update_option( 'sa_wc_gateways_by_location', $gateway_by_locations );
				}

			} else {
				update_option( 'sa_wc_gateways_by_location', '' );
			}

			if ( array_key_exists('sa_wc_gateways_by_location_search_in', $_POST) ) {
				update_option( 'sa_wc_gateways_by_location_search_in', $_POST['sa_wc_gateways_by_location_search_in'] );
			} else {
				update_option( 'sa_wc_gateways_by_location_search_in', '' );
			}
		}

		function plugin_action_links( $links ) {
			$action_links = array(
				'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout' ) . '" title="' . esc_attr( __( 'View Settings', 'woocommerce-gateways-by-location' ) ) . '">' . __( 'Settings', 'woocommerce-gateways-by-location' ) . '</a>',
			);

			return array_merge( $action_links, $links );
		}

		/**
		 * Function to show admin notice that WooCommerce Gateways By Location works with WC 2.5+
		 */
		function admin_notice_sa_needs_wc_25_above() { 
			?>
			<div class="updated error">
				<p><?php
					echo sprintf(__( '%s WooCommerce Gateways By Location is active but it will only work with WooCommerce 2.5+. %s.', 'woocommerce-gateways-by-location' ), '<strong>' . __( 'Important', 'woocommerce-gateways-by-location' ) . ':</strong>', '<a href="'.admin_url('plugins.php?plugin_status=upgrade').'" target="_blank" >' . __( 'Please update WooCommerce to the latest version', 'woocommerce-gateways-by-location' ) . '</a>' );
				?></p>
			</div>
			<?php
		}
	}
}
